import axios, { AxiosRequestConfig } from "axios";

// Define the types for the input data and the response data
type InputData = any;
type ResponseData = any;

// Define the API client class
export class APILibrary {
  private readonly apiUrl: string;
  private readonly endpoints: Record<string, string>;
  private accessToken: string | null;

  constructor(
    environment: string,
    accessToken: string | null,
    endpoints: Record<string, string>
  ) {
    // Determine the API URL based on the environment
    //this.apiUrl = import.meta.env.APP_URL + "/api/proxy";
    this.apiUrl = "http://localhost:3000/api/proxy";

    // Set Access Token
    this.accessToken = accessToken;

    // Store the API endpoints
    this.endpoints = endpoints;
  }

  // Method to send GET request to the API endpoint by name
  async get(
    endpointName: string,
    inputData?: InputData
  ): Promise<ResponseData> {
    return this.sendRequest("get", endpointName, inputData);
  }

  // Method to send POST request to the API endpoint by name
  async post(
    endpointName: string,
    inputData: InputData
  ): Promise<ResponseData> {
    return await this.sendRequest("post", endpointName, inputData);
  }

  // Method to send PUT request to the API endpoint by name
  async put(endpointName: string, inputData: InputData): Promise<ResponseData> {
    return this.sendRequest("put", endpointName, inputData);
  }

  // Method to send DELETE request to the API endpoint by name
  async delete(
    endpointName: string,
    inputData: InputData
  ): Promise<ResponseData> {
    return this.sendRequest("delete", endpointName, inputData);
  }

  // Method to send PATCH request to the API endpoint by name
  async patch(
    endpointName: string,
    inputData: InputData
  ): Promise<ResponseData> {
    return this.sendRequest("patch", endpointName, inputData);
  }

  addToken(token: string) {
    this.accessToken = token;
  }

  // Generic method to send HTTP request to the API endpoint by name
  private async sendRequest(
    method: string,
    endpointName: string,
    inputData: InputData
  ): Promise<ResponseData> {
    // Find the API endpoint by name
    const endpoint = this.endpoints[endpointName];
    if (!endpoint) {
      throw new Error(`API endpoint not found for name: ${endpointName}`);
    }

    // Prepare the request config
    const config: AxiosRequestConfig = {
      method,
      url: `${this.apiUrl}${endpoint}`,
      data: inputData,
      headers: {
        "Content-Type": "application/json",
        Accept: "application/json",
        // Send authorization token as a cookie or JWT based on accessToken presence
        ...(this.accessToken && {
          Authorization: `Bearer ${this.accessToken}`,
        }),
      },
    };

    // Send the HTTP request using axios
    const response = await axios(config);

    // Return the response data
    return response.data;
  }
}

// Define the union type for API endpoints
type ApiEndpoint =
  | "aa" // GET, POST, DELETE
  | "aa_add" // POST
  | "active_calls" // GET
  | "active_registrations" // GET
  | "alias" // GET, POST, DELETE
  | "alias_add" // POST
  | "announcement" // GET, POST, DELETE
  | "announcement_add" // POST
  | "auth_login" // POST
  | "auth_whoami" // GET
  | "billing_price_plan" // GET, POST, DELETE
  | "billing_price_plan_user" // GET
  | "billing_price_plan_add" // POST
  | "billing_recording_pricing" // GET, POST, DELETE
  | "billing_recording_pricing_add" // POST
  | "cid_enforcement" // GET
  | "contact" // GET, POST, DELETE
  | "contact_add" // POST
  | "directory_domain" // GET, POST, DELETE
  | "directory_domain_add" // POST
  | "directory_extension" // GET, POST, DELETE
  | "directory_extension_password" // GET
  | "directory_extension_add" // POST
  | "domain" // GET, POST, DELETE
  | "domain_add" // POST
  | "fifo_queue" // GET, POST, DELETE
  | "fifo_queue_add" // POST
  | "group" // GET, POST, DELETE
  | "group_add" // POST
  | "line_group" // GET, POST, DELETE
  | "line_group_add" // POST
  | "list_all" // GET
  | "list_all_count" // GET
  | "list_all_destinations" // GET
  | "manual_switch" // GET, POST, DELETE
  | "manual_switch_add" // POST
  | "provisioning_add_grandstream" // POST
  | "provisioning_list_grandstream" // GET
  | "queue_queue" // GET, POST, DELETE
  | "queue_queue_add" // POST
  | "sounds" // GET, POST, DELETE
  | "timeswitch" // GET, POST, DELETE
  | "timeswitch_add" // POST
  | "cgrt_client"; // GET

// Define the list of API endpoints with corresponding names based on URLs
const apiEndpoints: Record<ApiEndpoint, string> = {
  aa: "/aa",
  aa_add: "/aa/add",
  active_calls: "/active/calls",
  active_registrations: "/active/registrations",
  alias: "/alias",
  alias_add: "/alias/add",
  announcement: "/announcement",
  announcement_add: "/announcement/add",
  auth_login: "/auth/login",
  auth_whoami: "/auth/whoami",
  billing_price_plan: "/billing/price_plan",
  billing_price_plan_user: "/billing/price_plan_user",
  billing_price_plan_add: "/billing/price_plan/add",
  billing_recording_pricing: "/billing/recording_pricing",
  billing_recording_pricing_add: "/billing/recording_pricing/add",
  cid_enforcement: "/cid_enforcement",
  contact: "/contact",
  contact_add: "/contact/add",
  directory_domain: "/directory/domain",
  directory_domain_add: "/directory/domain/add",
  directory_extension: "/directory/extension",
  directory_extension_password: "/directory/extension/{extn_no}/password_ha1",
  directory_extension_add: "/directory/extension/add",
  domain: "/domain",
  domain_add: "/domain/add",
  fifo_queue: "/fifo/queue",
  fifo_queue_add: "/fifo/queue/add",
  group: "/group",
  group_add: "/group/add",
  line_group: "/line/group",
  line_group_add: "/line/group/add",
  list_all: "/list_all",
  list_all_count: "/list_all/count",
  list_all_destinations: "/list_all/destinations",
  manual_switch: "/manual_switch",
  manual_switch_add: "/manual_switch/add",
  provisioning_add_grandstream: "/provisioning/add_grandstream",
  provisioning_list_grandstream: "/provisioning/list_grandstream",
  queue_queue: "/queue/queue",
  queue_queue_add: "/queue/queue/add",
  sounds: "/sounds",
  timeswitch: "/timeswitch",
  timeswitch_add: "/timeswitch/add",
  cgrt_client: "/cgrt/client",
};

// Export the API client instance
export const api = new APILibrary(
  import.meta.env.API_ENVIRONMENT ?? "development",
  null,
  apiEndpoints
);
